/**
 *  Copyright (C) (2010-2012) Vadim Biktashev, Alexander Karpov, 
 *                            Irina Biktasheva, Ross McFarlane;
 *  Copyright (C) 2012 Sanjay Kharche
 *
 *  This file is part of Beatbox.
 *
 *   Beatbox is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   Beatbox is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with Beatbox.  If not, see <http://www.gnu.org/licenses/>.
 */


/* Linearise Cubic FitzHugh-Nagumo model in Winfree's notation [Chaos1(3):305,1991] */
#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "system.h"
#include "beatbox.h"
#include "FHNCUB.on"

#if FHNCUB

#include "device.h"
#include "state.h"
#include "bikt.h"
#include "rhs.h"
#include "qpp.h"

typedef struct {
  real
    bet, gam, eps;		/* Winfree's Kinetics A parameters */
  real
    U;                          /* u-component of the unperturbed solution */
  real 
    Iu,  Iv; 			/* External influence */
} STR;

RHS_HEAD(fhnl,2) {
  DEVICE_CONST(real,bet)
  DEVICE_CONST(real,gam)
  DEVICE_CONST(real,eps)
  DEVICE_CONST(real,Iu)
  DEVICE_CONST(real,Iv)
  DEVICE_CONST(real,U)
  real u1, v1;
  u1 = u[0];
  v1 = u[1];
  du[0] = ((1-U*U)*u1-v1)/eps + Iu;
  du[1] = (u1-gam*v1)*eps + Iv;
} RHS_TAIL(fhnl)

RHS_CREATE_HEAD(fhnl)
  ACCEPTP(eps,RNONE,RSUCC(0.),RNONE);
  ACCEPTP(bet,RNONE,RSUCC(0.),RNONE);
  ACCEPTP(gam,RNONE,RSUCC(0.),RNONE);
  ACCEPTP(U,RNONE,RNONE,RNONE);
  ACCEPTP(Iu,0,RNONE,RNONE);
  ACCEPTP(Iv,0,RNONE,RNONE);
RHS_CREATE_TAIL(fhnl,2)

#endif
