/**
 *  Copyright (C) (2010-2012) Vadim Biktashev, Alexander Karpov, 
 *                            Irina Biktasheva, Ross McFarlane;
 *  Copyright (C) 2012 Sanjay Kharche
 *
 *  This file is part of Beatbox.
 *
 *   Beatbox is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   Beatbox is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with Beatbox.  If not, see <http://www.gnu.org/licenses/>.
 */


/* response functions (finally) for Cubic FitzHugh-Nagumo 
 * model in Winfree's notation [Chaos1(3):305,1991],
 * with some more parameters added
 * SPACE MODE
 */
#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "system.h"
#include "beatbox.h"

#include "device.h"
#include "state.h"
#include "bikt.h"
#include "rhs.h"
#include "qpp.h"

typedef struct {
  real
    w, 
    bet, gam,			/* Winfree's Kinetics A parameters */
    epsu, epsv;			/* may be required different */
  real 
    Iu,  Iv, Iu1, Iv1, UU; 			/* External influence */
} STR;

RHS_HEAD(fhnresfs,4) {
  DEVICE_CONST(real,bet)
  DEVICE_CONST(real,gam)
  DEVICE_CONST(real,epsu)
  DEVICE_CONST(real,epsv)
  DEVICE_CONST(real,Iu)
  DEVICE_CONST(real,Iv)
  DEVICE_CONST(real,Iu1)
  DEVICE_CONST(real,Iv1)
  DEVICE_CONST(real,w)
  DEVICE_CONST(real,UU)
  real U, V, U2, U1, V1;
  U = u[0];
  V = u[1];
  U1 = u[2];
  V1 = u[3];
  U2 = UU*UU;
  du[0] = U*(1-U2)/epsu + epsv*U1 + Iu + w*V;
  du[1] = V*(1-U2)/epsu + epsv*V1 + Iv - w*U;
  du[2] = -U/epsu - epsv*gam*U1 + Iu1 + w*V1;
  du[3] = -V/epsu - epsv*gam*V1 + Iv1 - w*U1;
} RHS_TAIL(fhnresfs)

RHS_CREATE_HEAD(fhnresfs)
  ACCEPTP(epsu,RNONE,RSUCC(0.),RNONE);
  ACCEPTP(epsv,RNONE,RSUCC(0.),RNONE);
  ACCEPTP(bet,RNONE,RSUCC(0.),RNONE);
  ACCEPTP(gam,RNONE,RSUCC(0.),RNONE);
  ACCEPTP(Iu,0,RNONE,RNONE);
  ACCEPTP(Iv,0,RNONE,RNONE);
  ACCEPTP(Iu1,0,RNONE,RNONE);
  ACCEPTP(Iv1,0,RNONE,RNONE);
  ACCEPTP(w,0,RNONE,RNONE);
  ACCEPTP(UU,0,RNONE,RNONE);
RHS_CREATE_TAIL(fhnresfs,4)
